function [F, cohv] = ISPM_cor_tol(m, n, desired_coh, alpha, Nit, tol_stop)

% Incoherence via Shifted Power Method
% Corrected version of the algorithm appeared in IEEE SPL, vol.24, Sept.2017
% "Designing Incoherent Frames With Only MatrixVector Multiplications".
% This version includes a stopping criterion, to take advantage of quick convergence.
%
% Input:
%   m,n         - frame size
%   desired_coh - desired coherence, used as target (if too small, the
%                 algorithm does not converge)
%   alpha       - trade-off parameter; should be 0.5 or larger
%   Nit         - number of iterations (the algorithm does not stop bases
%                 on an error criterion
%   tol_stop    - stopping tolerance
% Output:
%   F           - the frame
%   cohv        - vector of mutual coherences at each iteration

% Note: the correction regards eq.(6): a square root should be applied on
% the right hand side

% BD 3.05.2018, 11.11.2022

% start with UNTF
F = gen_rand_untf(m,n);

cohv = zeros(1,Nit);
coh_min = 1;

% main loop
for k = 1 : Nit
  perm = randperm(n-1)+1;
  for j = perm  % a round of the atoms in random order
    d = F(:,j);
    v = F'*d;
    v(j) = 0;
    w = max(abs(v)/desired_coh, 1);
    w_max = max(w);
    eig_shift = (n/m)*alpha*w_max + (1-alpha)*sum(w)/m; % compute shift
    d = F*(w.*v) - eig_shift*d; % power method iteration
    d = d / norm(d);
    F(:,j) = d;
  end
  cohv(k) = max(max(abs(F'*F - eye(n))));
  if cohv(k) < coh_min
    coh_min = cohv(k);
    F_best = F;
  end

  if coh_min - desired_coh < tol_stop  % if coherence is near enough the target, stop
    cohv = cohv(1:k);
    break
  end

end

% make sure that it is UNTF (most likely useless)
F = F_best;
%[U,~,V] = svd(F);
%F = U*V(:,1:m)'*sqrt(n/m);
